package states 
{
	import com.greensock.TweenLite;
	import com.greensock.TweenMax;
	import core.Assets;
	import core.Game;
	import flash.geom.Rectangle;
	import flash.media.SoundChannel;
	import flash.utils.getTimer;
	import interfaces.IState;
	import managers.CollisionManager;
	import managers.FruitsManager;
	import managers.ParticleManager;
	import objects.BgParallax;
	import objects.Dog;
	import objects.BadBirds;
	import objects.DogLost;
	import starling.core.Starling;
	import starling.display.Image;
	import starling.display.MovieClip;
	import starling.display.Sprite;
	import starling.events.EnterFrameEvent;
	import starling.events.Event;
	import starling.events.Touch;
	import starling.events.TouchEvent;
	import starling.events.TouchPhase;
	import starling.text.TextField;
	
	/**
	 * ...
	 * @author miyijura
	 */
	public class Play extends Sprite implements IState 
	{
		public var husky:Dog;
		private var bg:BgParallax;
		public var game:Game;
		
		//game timer
		private var gameTimer:uint = getTimer();
		private var elapsedTime:uint;
		

		//bad birds
		private var badBird1:BadBirds = new BadBirds(Assets.ta.getTextures("redBird_"), 12);
		private var badBird2:BadBirds = new BadBirds(Assets.ta.getTextures("yellowBird_"), 12);
		private var badBird3:BadBirds = new BadBirds(Assets.ta.getTextures("greyBird_"), 12);
		private var badBird4:BadBirds = new BadBirds(Assets.ta.getTextures("blueBird_"), 12);
		private var myAlert:Image;
		private var explosion:MovieClip;
		private var collisionManager:CollisionManager;
		public var fruitManager:FruitsManager;
		public var particleManager:ParticleManager;
		
		//para contar las colisiones del husky
		private var hitPoints:uint = 0;
		
		//el score!
		public var myScore:TextField = new TextField(200, 100, "SCORE "+ myPoints as String);
		public var myPoints:Number = 0;
		
		private var myChannel:SoundChannel = new SoundChannel();
		private var dogLostScreen:DogLost;
		
		//public var gameArea:Rectangle;
		
		public function Play(game:Game) 
		{
			this.game = game;
			
			addEventListener(Event.ADDED_TO_STAGE, init);
			
		}
		
		private function init(e:Event):void 
		{
			removeEventListener(Event.ADDED_TO_STAGE, init);
			trace("PLAY!");
			
			//para crear un area de juego por medio de un rectangulo
			//gameArea = new Rectangle(0, 100, stage.stageWidth, stage.stageHeight - 250);
			
			//musica!
			myChannel = Assets.play_music.play(0, 9999);
			
			//inicializamos el timer
			gameTimer = 0;
			elapsedTime = 0;
			
			addEventListener(EnterFrameEvent.ENTER_FRAME, checkTimer);
			
			//creamos el bg
			bg = new BgParallax();
			//bg.speed = 10;
			addChild(bg);
			
			//creamos al husky
			husky = new Dog(Assets.ta.getTextures("husky_run_"), 12);
			husky.x = 400;
			husky.y = 400;
			//volteamos al husky hacia la derecha
			husky.scaleX = -1;
			Starling.juggler.add(husky);
			addChild(husky);
			
			//add enemies
			//recuerda que debes de pasar referencia a esta pantalla de juego con this
			createEnemies();
			
			//add fruits!
			fruitManager = new FruitsManager(this);
			//collision manager
			collisionManager = new CollisionManager(this);
			//particles manager
			particleManager = new ParticleManager(this);
			
			//creamos el campo de texto del score
			
			myScore.format.size = 60;
			myScore.format.font = "myFont";
			myScore.format.color = 0x000000;
			myScore.x = stage.stageWidth / 2 - myScore.width / 2;
			myScore.y = 10;
			addChild(myScore);
			
			//add alert sign
			myAlert = new Image(Assets.ta.getTexture("warning-01.png"));
			//colocamos el pivote al centro para una mejor transformacion
			myAlert.pivotX = myAlert.width * 0.5;
			myAlert.pivotY  = myAlert.height * 0.5;
			//escalamos el objeto a 0 para desaparecerlo momentaneamente
			myAlert.scaleX = myAlert.scaleY = 0;
			addChild(myAlert);
			//posicionamos el warning 45 pixeles de la orilla derecha
			myAlert.x = stage.stageWidth - 45;
			
			//creamos la explosion
			explosion = new MovieClip(Assets.ta.getTextures("explosion_"), 12);
			explosion.pivotX = explosion.width * 0.5;
			explosion.pivotY  = explosion.height * 0.5;
			addChild(explosion);
			explosion.visible = false;
			explosion.loop = false;
			
			//dog lost screen
			dogLostScreen = new DogLost();
			//dog lost screen empieza invisible
			addChild(dogLostScreen);
			dogLostScreen.alpha = 0;
			
			addEventListener(TouchEvent.TOUCH, dogMove);
		}
		
		private function checkTimer(e:EnterFrameEvent):void 
		{
			//actualizamos el score con los puntos detectados por collision manager
			myPoints = collisionManager.scorePoints;
			myScore.text = "SCORE " + myPoints;
			
			//empezamos a calcular el tiempo que ha transcurrido
			elapsedTime = gameTimer++;	
			//evaluamos si paso el tiempo que queremos para activar los enemigos
			if (elapsedTime >= 200) {
				gameTimer = 0;
				//esta variable nos permite sacar un numero aleatorio del 0 al 400
				var aBird:uint = Math.ceil(Math.random() * 400);
				trace ("aBird:" + aBird);
				//lo pasamos como parametro para solicitar algun tipo de pajaro entre los 4 disponibles
				activateBirds(aBird);
			}
			//test trace
			//trace("duration: " + Math.ceil(gameTimer));
		}
		
		private function activateBirds(aBird:uint):void 
		{
			//evaluamos el valor de aBird para activar los pajaros requeridos
			//modificamos el valor de attackSpeed para que la velocidad de los pajaros sea distinta
			if (aBird <= 100) {
				badBird1.y = Math.ceil(Math.random() * 550);
				myAlert.y = badBird1.y;
				TweenLite.to(myAlert, 0.5, {scaleX:0.3, scaleY:0.3, onComplete:hideAlert } );
				badBird1.attackSpeed = 1.5;
				badBird1.attackWave();
			} else if (aBird >= 101 && aBird <= 200) { //si esta entre 101 y 200
				badBird2.y = Math.ceil(Math.random() * 550);
				myAlert.y = badBird2.y;
				TweenLite.to(myAlert, 0.5, {scaleX:0.3, scaleY:0.3, onComplete:hideAlert } );
				badBird2.attackSpeed = 1.8;
				badBird2.attackWave();
			} else if (aBird >= 201 && aBird <= 300) {
				badBird3.y = Math.ceil(Math.random() * 550);
				myAlert.y = badBird3.y;
				TweenLite.to(myAlert, 0.5, {scaleX:0.3, scaleY:0.3, onComplete:hideAlert } );
				badBird3.attackSpeed = 1.7;
				badBird3.attackWave();
			} else if (aBird >= 301 && aBird <= 400) {
				badBird4.y = Math.ceil(Math.random() * 550);
				myAlert.y = badBird4.y;
				TweenLite.to(myAlert, 0.5, {scaleX:0.3, scaleY:0.3, onComplete:hideAlert } );
				badBird4.attackSpeed = 2;
				badBird4.attackWave();
			}
			
			addEventListener(EnterFrameEvent.ENTER_FRAME, checkCollisions);
		}
		
		private function checkCollisions(e:Event):void 
		{
			if(badBird1.bounds.intersects(husky.bounds))
			{
				//affect screeen!
				TweenLite.to(this, 0.5, { scaleX:1.2, scaleY:1.2, onComplete:stabilizeScreen } );
				
				explosion.x = badBird1.x;
				explosion.y = badBird1.y;
				TweenLite.killTweensOf(badBird1);
				badBird1.reuseBirds();
				explosion.visible = true;
				explosion.currentFrame = 0;
				Starling.juggler.add(explosion);
				Assets.crash_sound.play();
				//contamos una colision mas cada que choca
				hitPoints++;
			} else if(badBird2.bounds.intersects(husky.bounds))
			{
				TweenLite.to(this, 0.5, { scaleX:1.2, scaleY:1.2, onComplete:stabilizeScreen } );
				explosion.x = badBird2.x;
				explosion.y = badBird2.y;
				TweenLite.killTweensOf(badBird2);
				badBird2.reuseBirds();
				explosion.visible = true;
				explosion.currentFrame = 0;
				Starling.juggler.add(explosion);
				Assets.crash_sound.play();
				//contamos una colision mas cada que choca
				hitPoints++;
			} else if(badBird3.bounds.intersects(husky.bounds))
			{
				TweenLite.to(this, 0.5, { scaleX:1.2, scaleY:1.2, onComplete:stabilizeScreen } );
				explosion.x = badBird3.x;
				explosion.y = badBird3.y;
				TweenLite.killTweensOf(badBird3);
				badBird3.reuseBirds();
				explosion.visible = true;
				explosion.currentFrame = 0;
				Starling.juggler.add(explosion);
				Assets.crash_sound.play();
				//contamos una colision mas cada que choca
				hitPoints++;
			} else if(badBird4.bounds.intersects(husky.bounds))
			{
				TweenLite.to(this, 0.5, { scaleX:1.2, scaleY:1.2, onComplete:stabilizeScreen } );
				explosion.x = badBird4.x;
				explosion.y = badBird4.y;
				TweenMax.killTweensOf(badBird4);
				badBird4.reuseBirds();
				explosion.visible = true;
				explosion.currentFrame = 0;
				Starling.juggler.add(explosion);
				Assets.crash_sound.play();
				//contamos una colision mas cada que choca
				hitPoints++;
			}
			
			//3 impactos y el husky muere!
			if (hitPoints >= 3) {
				huskyDies();
			}
			
			explosion.addEventListener(Event.COMPLETE, explosionComplete);
		}
		
		private function huskyDies():void 
		{
			//movemos al husky fuera de la pantalla
			husky.y = -100;
			husky.x = 850;
			//movemos la pantalla dog lost a un nivel superior
			this.setChildIndex(dogLostScreen, this.numChildren - 1);
			//quitamos el evento que mueve a husky
			removeEventListener(TouchEvent.TOUCH, dogMove);
			//aparecemos una pantalla dog lost
			TweenLite.to(dogLostScreen, 1, { alpha:0.7 } );
			
			if (gameTimer == 200)
			gameover();
		}
		
		private function stabilizeScreen():void 
		{
			TweenLite.to(this, 0.5, { scaleX:1, scaleY:1 } );

		}
		
		private function explosionComplete(e:Event):void 
		{
			explosion.removeEventListener(Event.COMPLETE, explosionComplete);
			explosion.visible  = false;
			Starling.juggler.remove(explosion);
			
		}
		
		//escondemos la alerta escalandola a 0
		private function hideAlert():void 
		{
			myAlert.scaleX = myAlert.scaleY = 0;
		}
		
		//create enemies
		private function createEnemies():void 
		{
			//solamente los agregamos al display list y los activamos con el juggler	
			Starling.juggler.add(badBird1);
			addChild(badBird1);
			trace("YEY");
		
			Starling.juggler.add(badBird2);
			addChild(badBird2);
			trace("YEY");
		
			Starling.juggler.add(badBird3);
			addChild(badBird3);
			trace("YEY");
		
			Starling.juggler.add(badBird4);
			addChild(badBird4);
			trace("YEY");
		}
		
		private function dogMove(e:TouchEvent):void 
		{
			var touch:Touch = e.getTouch(this, TouchPhase.HOVER);
			if (touch)
			{
				TweenLite.to(husky, 1, { x:touch.globalX - 20,  y:touch.globalY - 20 } );
			}
		}
		
		/* INTERFACE interfaces.IState */
		
		public function update():void 
		{
			bg.update();
			fruitManager.update();
			collisionManager.update();
			
		}
		
		private function gameover():void {
			//llamamos al estado de juego GAME_OVER de la clase Game
			game.changeState(Game.GAME_OVER_STATE);
		}
		
		
		public function destroy():void 
		{
			removeEventListener(TouchEvent.TOUCH, dogMove);
			removeEventListener(EnterFrameEvent.ENTER_FRAME, checkCollisions);
			removeEventListener(EnterFrameEvent.ENTER_FRAME, checkTimer);
			//detenemos la musica
			myChannel.stop();
			removeFromParent(true);
			
		}
		
	}

}